from __future__ import unicode_literals

from shortuuid import uuid

from django import template
from django.conf import settings
from django.contrib.auth.models import User
from django.core.mail import send_mail
from django.core.urlresolvers import reverse
from django.utils.http import int_to_base36

from .models import OptOut
from .views import import_attribute, TOKEN_GENERATOR


FROM_EMAIL = getattr(settings, 'INVITER_FROM_EMAIL',
                     settings.DEFAULT_FROM_EMAIL)

token_generator = import_attribute(TOKEN_GENERATOR)


def send_invite(invitee, inviter, url=None, opt_out_url=None, **kwargs):
    """
    Send the default invitation email assembled from
    ``inviter2/email/subject.txt`` and ``inviter2/email/body.txt``

    Both templates will receive all the ``kwargs``.

    :param invitee: The invited user
    :param inviter: The inviting user
    :param url: The invite URL
    :param subject_template: The template to render for the subject
    :param body_template: The template to render for the body
    :param opt_out_url: A URL where users can permanently opt out of
                        invitations
    """
    ctx = {'invitee': invitee, 'inviter': inviter}
    ctx.update(kwargs)
    ctx.update(url=url)
    ctx.update(opt_out_url=opt_out_url)
    ctx = template.Context(ctx)

    subject_template = kwargs.pop('subject_template',
                                  'inviter2/email/subject.txt')
    body_template = kwargs.pop('body_template', 'inviter2/email/body.txt')

    subject = template.loader.get_template(subject_template)
    body = template.loader.get_template(body_template)

    subject = subject.render(ctx)
    body = body.render(ctx)

    # Newlines in subject lines are not allowed
    subject = ' '.join(subject.split('\n'))

    send_mail(subject, body, FROM_EMAIL, [invitee.email])


def invite(email, inviter, sendfn=send_invite, resend=True, **kwargs):
    """
    Invite a given email address and return a ``(User, sent)`` tuple similar
    to the Django :meth:`django.db.models.Manager.get_or_create` method.

    If a user with ``email`` address does not exist:

    * Creates a :class:`django.contrib.auth.models.User` object
    * Set ``user.email = email``
    * Set ``user.is_active = False``
    * Set a random password
    * Send the invitation email
    * Return ``(user, True)``

    If a user with ``email`` address exists and ``user.is_active == False``:

    * Re-send the invitation
    * Return ``(user, True)``

    If a user with ``email`` address exists:

    * Don't send  the invitation
    * Return ``(user, False)``

    If the email address is blocked:

    * Don't send the invitation
    * Return ``(None, False)``

    To customize sending, pass in a new ``sendfn`` function as documented by
    :attr:`inviter2.utils.send_invite`:

    ::

        sendfn = lambda invitee, inviter, **kwargs: 1
        invite("foo@bar.com", request.user, sendfn = sendfn)


    :param email: The email address
    :param inviter: The user inviting the email address
    :param sendfn: An email sending function. Defaults to
                   :attr:`inviter2.utils.send_invite`
    :param resend: Resend email to users that are not registered yet
    """

    if OptOut.objects.is_blocked(email):
        return None, False
    try:
        user = User.objects.get(email=email)
        if user.is_active:
            return user, False
        if not resend:
            return user, False
    except User.DoesNotExist:
        user = User.objects.create(
            username=uuid(),
            email=email,
            is_active=False
        )
        user.set_unusable_password()
        user.save()

    url_parts = int_to_base36(user.id), token_generator.make_token(user)
    url = reverse('inviter2:register', args=url_parts)

    opt_out_url = reverse('inviter2:opt-out', args=url_parts)
    kwargs.update(opt_out_url=opt_out_url)

    sendfn(user, inviter, url=url, **kwargs)
    return user, True



"""
This file demonstrates writing tests using the unittest module. These will pass
when you run "manage.py test".

Replace this with more appropriate tests for your application.
"""
from __future__ import unicode_literals

from shortuuid import uuid

from six.moves.urllib.parse import urlparse

from django.contrib.auth.models import User
from django.core import mail
from django.core.urlresolvers import reverse
from django.http import Http404
from django.test import TestCase
from django.utils.http import int_to_base36

from .models import OptOut
from .utils import invite, token_generator
from .views import UserMixin


class InviteTest(TestCase):
    def setUp(self):
        self.inviter = User.objects.create(username=uuid())
        self.existing = User.objects.create(username=uuid(),
                                            email='existing@example.com')

    def test_inviting(self):
        user, sent = invite("foo@example.com", self.inviter)
        self.assertTrue(sent)
        self.assertFalse(user.is_active)
        self.assertEqual(1, len(mail.outbox))
        self.assertEqual(3, User.objects.count())

        # Resend the mail
        user, sent = invite("foo@example.com", self.inviter)
        self.assertTrue(sent)
        self.assertFalse(user.is_active)
        self.assertEqual(2, len(mail.outbox))
        self.assertEqual(3, User.objects.count())

        # Don't resend the mail
        user, sent = invite("foo@example.com", self.inviter, resend=False)
        self.assertFalse(sent)
        self.assertFalse(user.is_active)
        self.assertEqual(2, len(mail.outbox))
        self.assertEqual(3, User.objects.count())

        # Don't send the email to active users
        user, sent = invite("existing@example.com", self.inviter)
        self.assertFalse(sent)
        self.assertTrue(user.is_active)
        self.assertEqual(2, len(mail.outbox))
        self.assertEqual(3, User.objects.count())

    def test_views(self):
        user, sent = invite("foo@example.com", self.inviter)
        self.assertTrue(sent)
        url_parts = int_to_base36(user.id), token_generator.make_token(user)

        url = reverse('inviter2:register', args=url_parts)

        resp = self.client.get(url)

        self.assertEqual(200, resp.status_code, resp.status_code)

        resp = self.client.post(url, {'username': 'testuser',
                                      'email': 'foo@example.com',
                                      'password1': 'test-1234',
                                      'password2': 'test-1234'})

        self.assertEqual(302, resp.status_code, resp.content)

        self.client.login(username='testuser', password='test-1234')

        resp = self.client.get(reverse('inviter2:done'))

        self.assertEqual(200, resp.status_code, resp.status_code)

    def test_error_views(self):
        # invalid base36 encoded user id
        user, sent = invite("foo@example.com", self.inviter)
        self.assertTrue(sent)
        url_parts = 'z'*13, token_generator.make_token(user)
        url = reverse('inviter2:register', args=url_parts)
        resp = self.client.get(url)
        self.assertEqual(404, resp.status_code, resp.status_code)

        # attempt to use some other user's id
        user, sent = invite("foo@example.com", self.inviter)
        self.assertTrue(sent)
        uid = int_to_base36(self.existing.id)
        token = token_generator.make_token(user)
        url = reverse('inviter2:register', args=(uid, token))
        resp = self.client.get(url)
        self.assertEqual(403, resp.status_code, resp.status_code)

        # submit an invalid form
        user, sent = invite("foo@example.com", self.inviter)
        self.assertTrue(sent)
        url_parts = int_to_base36(user.id), token_generator.make_token(user)
        url = reverse('inviter2:register', args=url_parts)
        resp = self.client.get(url)
        self.assertEqual(200, resp.status_code, resp.status_code)
        resp = self.client.post(url, {'username': 'testuser',
                                      'email': 'foo@example.com',
                                      'password1': 'test-1234',
                                      'password2': 'test-4321'})
        self.assertEqual(200, resp.status_code, resp.content)
        self.assertIn(
            'The two password fields didn&#39;t match.', str(resp.content))

        # developer with bad redirect URL
        with self.settings(INVITER_REDIRECT='http://example.com/'):
            resp = self.client.post(url, {'username': 'testuser',
                                          'email': 'foo@example.com',
                                          'password1': 'test-1234',
                                          'password2': 'test-1234'})
            self.assertEqual(302, resp.status_code, resp.content)
            self.assertEqual(resp['Location'], 'http://example.com/')

    def test_get_user(self):
        mixin = UserMixin()
        with self.assertRaises(Http404) as e:
            mixin.get_user('z'*14)
        self.assertEqual(str(e.exception), 'No such invited user.')

    def test_opt_out(self):
        self.assertEqual(2, User.objects.count())

        user, sent = invite("foo@example.com", self.inviter)
        self.assertTrue(sent)
        self.assertEqual(1, len(mail.outbox))
        self.assertEqual(3, User.objects.count())

        url_parts = int_to_base36(user.id), token_generator.make_token(user)
        url = reverse('inviter2:opt-out', args=url_parts)

        resp = self.client.get(url)
        self.assertEqual(200, resp.status_code, resp.status_code)

        resp = self.client.post(url, {})
        self.assertEqual(302, resp.status_code, resp.status_code)
        self.assertEqual(reverse('inviter2:opt-out-done'),
                         urlparse(resp['Location']).path)
        self.assertEqual(2, User.objects.count())

        user, sent = invite("foo@example.com", self.inviter)
        self.assertFalse(sent)
        self.assertEqual(1, len(mail.outbox))
        self.assertEqual(1, OptOut.objects.count())
        self.assertTrue(OptOut.objects.is_blocked("foo@example.com"))
        self.assertIsNone(user)

        opt_out = OptOut.objects.get()
        opt_hash = OptOut.objects._hash_email('foo@example.com')
        self.assertEqual(opt_out.__unicode__(), opt_hash)

    def test_opt_out_done(self):
        resp = self.client.get(reverse('inviter2:opt-out-done'))
        self.assertEqual(200, resp.status_code, resp.status_code)



from __future__ import unicode_literals

from django.conf import settings
from django.utils.crypto import salted_hmac, constant_time_compare


class TokenGenerator(object):
    def make_token(self, user):
        return self._make_token(user)

    def _make_token(self, user):
        value = '{id}-{last_login}-{password}'.format(
            id=user.id, last_login=user.last_login, password=user.password)
        return salted_hmac(settings.SECRET_KEY, value).hexdigest()[::2]

    def check_token(self, user, token):
        return constant_time_compare(token, self._make_token(user))

generator = TokenGenerator()



from __future__ import unicode_literals

from django.conf.urls import patterns, url

from .views import Register, Done, OptOut, OptOutDone

urlpatterns = patterns(
    '',
    url(r'^done/$', Done.as_view(), name='done'),
    url(r'^register/(?P<uidb36>[0-9A-Za-z]{1,13})-(?P<token>[0-9A-Za-z]+)/$',
        Register.as_view(), name='register'),
    url(r'^optout/(?P<uidb36>[0-9A-Za-z]{1,13})-(?P<token>[0-9A-Za-z]+)/$',
        OptOut.as_view(), name='opt-out'),
    url(r'^optout/done/$',
        OptOutDone.as_view(), name='opt-out-done'),
)
"""
.. attribute:: ^done/$

    :name: done

.. attribute:: ^register/(?P<uidb36>[0-9A-Za-z]{1,13})-(?P<token>[0-9A-Za-z]+)/$

    :name: register

.. attribute:: ^optout/(?P<uidb36>[0-9A-Za-z]{1,13})-(?P<token>[0-9A-Za-z]+)/$

    :name: opt-out

.. attribute:: ^optout/done/$

    :name: opt-out-done
"""
